local f = InfoPanel_AddPlugin();
local UPDATE_INTERVAL = 1;
local nextUpdate = 0;
local addons = {};
local fps, down, up, latency, mem;

--------------------------------------------------------------------------------------------------------
--                                             Functions                                              --
--------------------------------------------------------------------------------------------------------

-- OnUpdate
local function OnUpdate(self,elapsed)
	nextUpdate = (nextUpdate - elapsed);
	if (nextUpdate <= 0) then
		fps = GetFramerate();
		down, up, latency = GetNetStats();
		mem = gcinfo();

		InfoPanel_SetText(f,"|cff00ff00%.1f  |cffffd100%dms|r  %.2f Mb",fps,latency,mem / 1024);
		nextUpdate = UPDATE_INTERVAL;
	end
end

-- Mouse OnEnter
local function OnEnter(self)
	if (not IsShiftKeyDown()) then
		InfoPanel_MakeTip(f,"Performance Info");

		GameTooltip:AddDoubleLine("Frame Rate",format("%.2f",fps),nil,nil,nil,1,1,1);

		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("Bandwidth Down",format("%.2f bytes/s",down * 1024),nil,nil,nil,1,1,1);
		GameTooltip:AddDoubleLine("Bandwidth Up",format("%.2f bytes/s",up * 1024),nil,nil,nil,1,1,1);
		GameTooltip:AddDoubleLine("Latency",format("%d ms",latency),nil,nil,nil,1,1,1);

		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine("UI Memory Usage",format("%d kb",mem),nil,nil,nil,1,1,1);
		GameTooltip:AddLine("Press Shift for Detailed Addon Info",0.5,0.5,0.5);
	else
		UpdateAddOnMemoryUsage();
		local memTotal = 0;
		for i = 1, GetNumAddOns() do
			if (not addons[i]) then
				addons[i] = {};
			end
			addons[i].mem = GetAddOnMemoryUsage(i);
			addons[i].name = GetAddOnInfo(i);
			memTotal = (memTotal + addons[i].mem);
		end
		table.sort(addons,function(a,b) return a.mem > b.mem end);

		InfoPanel_MakeTip(f,"Addon Memory Usage ("..#addons..")");
		for index, addon in ipairs(addons) do
			if (addon.mem > 0) then
				GameTooltip:AddDoubleLine("|cffffffff"..format("%.2d|r ",index)..addon.name,format("%.2f kb",addon.mem),nil,nil,nil,1,1,1);
			end
		end
		GameTooltip:AddLine(strrep("-",58),1,1,1);
		GameTooltip:AddDoubleLine("Addon Memory Usage",format("%d kb",memTotal),nil,nil,nil,1,1,1);
		GameTooltip:AddDoubleLine("UI Memory Usage",format("%d kb",mem),nil,nil,nil,1,1,1);
	end

	GameTooltip:Show();
end

-- Perform a GC
local function PerformGC()
	UpdateAddOnMemoryUsage();
	local memBefore = gcinfo();
	collectgarbage();
	UpdateAddOnMemoryUsage();
	local memAfter = gcinfo();
	AzMsg(("|2Performed GC|r Before: |1%d|r kb, After: |1%d|r kb, Reduction: |1%d|r kb."):format(memBefore,memAfter,memBefore-memAfter));
end

--------------------------------------------------------------------------------------------------------
--                                              Finalize                                              --
--------------------------------------------------------------------------------------------------------

f.icon:SetTexture("Interface\\Icons\\Ability_Hunter_Pathfinding");
f:SetScript("OnEnter",OnEnter);
f:SetScript("OnUpdate",OnUpdate);
f:SetScript("OnClick",PerformGC);